#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 Axigen Messaging SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" Definitions of classes related to CLI Commands
    Command
      SimpleCommand
        BackCommand
        CommitCommand
      UpdateCommand
      ListCommand
      ShowCommand
      ConfigCommand    
    Macro
"""

__copyright__ = "Copyright (C) 2020, Axigen Messaging SRL"
__license__ = "Apache License Version 2.0"
__version__ = "1.1.0"
__author__ = 'eugen.belea@axigen.com (Eugen Belea)'
__maintainer__ = 'eugen.belea@axigen.com (Eugen Belea)'

try:
    import exceptions
except ImportError:
    import builtins as exceptions
import re
import string

from . import clierror
from . import cliconnection
#import axicli.handler

#PattCmdResponseStatus = re.compile('(?P<stat>(\+OK)|(\-ERR)): (?P<msg>.*)\r\n\<(?P<ctx>[A-Za-z0-9\-]*#?)\> ');


class Command():
    '''
    CLI Command (base class of all commands)
    '''
    (NAV_NONE, NAV_FW, NAV_BW) = (0, 1, -1)
    def __init__(self, _name=None, _nav=NAV_NONE):
        self.name = _name
        self.nav = _nav
        self.statOk = False
        self.statMsg = ""
        self.prompt = ""

    def getRequest(self):
        raise exceptions.NotImplementedError()

    def __navigate(self, ctxStack, log=None):
        if not self.statOk:
            return
        if self.nav == self.NAV_FW:
            ctxStack.push(self.prompt)
        elif self.nav == self.NAV_BW:
            topCtx = ctxStack.pop()
            if ctxStack.front() != self.prompt and log:
                log.warning("unexpected return context '%s' instead of '%s'", \
                    self.prompt, topCtx)
            del topCtx

    def __processResponse(self, conn, hnd=None):
        needRecv = True
        statDone = False
        line = ''
        lineNo = 1
        sio = []
        endWithCRLF = False
        while True:
            if needRecv:
                s = conn.recvResponse()
                if s == None: 
                    ''' connection closed '''
                    raise clierror.ConnectionClosedError(self.name)
#                    return False
                sio = (line + s).split('\r\n')
                endWithCRLF = (line + s).endswith('\r\n')
                needRecv = False
            if (len(sio) == 0):
                ''' buffered completely consumed '''
                needRecv = True
                continue
            line = sio[0]
            del sio[0]
            if endWithCRLF or len(sio) > 0:
                line = line + '\r\n'
                mo = cliconnection.PattCmdResponseStatus.match(line)
                if mo:
                    self.statMsg = mo.group('msg')
                    if mo.group('stat') == "+OK":
                        self.statOk = True
                        if hnd:
                            hnd.handleOk(self.statMsg)
                    else:
                        self.statOk = False
                        if hnd:
                            hnd.handleError(self.statMsg)
                    statDone = True
                else:
                    if hnd:
                        hnd.handleLine(line.rstrip('\r\n'), lineNo)
                lineNo = lineNo + 1
            else:
                if statDone:
                    mo = cliconnection.PattCmdResponsePrompt.match(line)
                    if mo:
                        self.prompt = mo.group('ctx')
                        break
                ''' new buffer to be read is needed '''
                needRecv = True
    
    def execute(self, conn, hnd=None, doSetLog=True):
        self.StatOk = True
        conn.sendRequest(self.getRequest())
        if hnd:
            if doSetLog:
                hnd.setLog(conn.log)
                self.__processResponse(conn, hnd)
                hnd.setLog()
            else:
                self.__processResponse(conn, hnd)
        else:
            self.__processResponse(conn)
        self.__navigate(conn.ctxStack, conn.log)
        return self.statOk

class Macro():
    '''
    CLI List of Command (macro of commands)
    '''
    def __init__(self, _cmdList=None):
        'Hello'
        self.cmdList = _cmdList
        self.statOk = True
        if self.cmdList:
            for cmd in self.cmdList:
                if not isinstance(cmd, Command):
                    raise exceptions.TypeError("axicli.clicommand.Command instance expected")
   
    def add(self, cmd):
        if not isinstance(cmd, Command):
            raise exceptions.TypeError("axicli.clicommand.Command instance expected")
        self.cmdList.append(cmd)

    def execute(self, conn, hnd=None):
        self.StatOk = True        
        if hnd:
            cmdNo = 1
            hnd.setLog(conn.log)
            for cmd in self.cmdList:
                hnd.handleCommandBegin(cmdNo, cmd.name)
                cmd.execute(conn, hnd, False)
                if not cmd.statOk:
                    self.statOk = False
                    break
                hnd.handleCommandEnd(cmdNo, cmd.name)
                cmdNo = cmdNo + 1
            hnd.setLog()
        else:
            for cmd in self.cmdList:
                cmd.execute(conn)
                if not cmd.statOk:
                    self.statOk = False
                    break
        return self.statOk

class SimpleCommand(Command):
    '''
    CLI one-word Simple Command
    '''
    def __init__(self, _cmd, _nav=Command.NAV_NONE):
        Command.__init__(self, _cmd.upper(), _nav)
        self.cmd = _cmd
    
    def getRequest(self):
        return self.cmd
    
class BackCommand(SimpleCommand):
    '''
    CLI Command for Navigation to previous context though Back
    '''
    def __init__(self):
        SimpleCommand.__init__(self, "BACK", Command.NAV_BW)

class CommitCommand(SimpleCommand):
    '''
    CLI Command for Navigation to previous context though Commit
    '''
    def __init__(self):
        SimpleCommand.__init__(self, "COMMIT", Command.NAV_BW)

class UpdateCommand(Command):
    def __init__(self, _objectType, _objectName):
        Command.__init__(self, "UPDATE " + _objectType, Command.NAV_FW);
        self.objectType = _objectType
        self.objectName = _objectName
    
    def getRequest(self):
        return "UPDATE %s name %s" % (self.objectType, self.objectName)

class ListCommand(Command):
    def __init__(self, _objectType):
        Command.__init__(self, "LIST " + _objectType)
        self.objectType = _objectType

    def getRequest(self):
        return "LIST " + self.objectType

class ShowCommand(Command):
    def __init__(self, _objectPart=None):
        self.objectPart = _objectPart
        if self.objectPart:
            Command.__init__(self, "SHOW " + self.objectPart)
        else:
            Command.__init__(self, "SHOW")
            
    def getRequest(self):
        if self.objectPart:
            return "SHOW " + self.objectPart
        else:
            return "SHOW"

class ConfigCommand(Command):
    def __init__(self, _objectPart):
        self.objectPart = _objectPart
        if self.objectPart:
            Command.__init__(self, "CONFIG " + self.objectPart, Command.NAV_FW)
        else:
            raise exceptions.AttributeError
            
    def getRequest(self):
        return "CONFIG " + self.objectPart
    
