#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 Axigen Messaging SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" Definitions of classes related to CLI Connections
    Connection
"""

__copyright__ = "Copyright (C) 2020, Axigen Messaging SRL"
__license__ = "Apache License Version 2.0"
__version__ = "1.1.0"
__author__ = 'eugen.belea@axigen.com (Eugen Belea)'
__maintainer__ = 'eugen.belea@axigen.com (Eugen Belea)'

try:
    import exceptions
except ImportError:
    import builtins as exceptions
import array
import io
import re
import socket
import logging
import sys

import axicli
from . import clierror
from .clicontext import *

PattLoginResponse = re.compile("(.*\r?\n)(.*\r?\n)\<(?P<ctx>[a-zA-Z]+)\> ?")
PattPasswordResponse = re.compile("\<(?P<ctx>[a-zA-Z]+)\> ")
PattRootResponse = re.compile("(.*\r?\n)(?P<stat>(\+OK)|(\-ERR))\: (?P<msg>.*)\r?\n\<(?P<ctx>[A-Za-z0-9\-]*\#?)\> ");
PattCmdResponseStatus = re.compile("(?P<stat>(\+OK)|(\-ERR)): (?P<msg>.*)\r?\n");
PattCmdResponsePrompt = re.compile("\<(?P<ctx>[A-Za-z0-9\-]*\#?)\> ")

CrtConnectionId = 0
ConnectionLoggerInit = False
LEVEL_PROTO = 8
#ConnectionLoggerLevel = logging.DEBUG #LEVEL_PROTO
#ConnectionLoggerLevel = LEVEL_PROTO
ConnectionLoggerLevel = logging.ERROR


def getConnectionId():
    global CrtConnectionId
    connId = CrtConnectionId
    CrtConnectionId = CrtConnectionId + 1
    return connId

def getConnectionLogger(connId):
    global LEVEL_PROTO, ConnectionLoggerInit, ConnectionLoggerLevel

    logger = logging.getLogger("CLI")
    if not ConnectionLoggerInit:
        logging.addLevelName(LEVEL_PROTO, "PROTO")
        logger.setLevel(ConnectionLoggerLevel)
        h = logging.StreamHandler()
        h.setLevel(ConnectionLoggerLevel)
        h.setFormatter(logging.Formatter("%(name)-12s%(levelname)-8s %(connId)08X: %(message)s"))
        logger.addHandler(h)
        ConnectionLoggerInit = True
    loggerAdapter = logging.LoggerAdapter(logger, { 'connId' : connId })
    return loggerAdapter

class Connection:
    (STAT_NEW, STAT_LOGGEDIN, STAT_LOGGEDOUT) = (0, 10, 20)
    BUFF_SZ = 4096 #1024
    def __init__ (self, _user, _passwd, _host="localhost", _port=7000):        
        ''' @var user username ''' 
        self.user = _user
        ''' @var passwd password '''
        self.passwd = _passwd
        ''' @var host ip address or hostname '''
        self.host = _host
        ''' @var port port '''
        self.port = _port
        ''' @var sock TCP socket '''        
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
#        self.sock.settimeout(1.0)
        ''' @var log logger '''
        self.id = getConnectionId()
        self.stat = self.STAT_NEW
        if sys.version_info[0] < 3:
          self.buf = array.array('c', '\0' * (self.BUFF_SZ + 1))
        else:
          self.buf = array.array('u', u'\0' * (self.BUFF_SZ + 1))
        self.log = getConnectionLogger(self.id)
        self.log.debug("created %s:%s@%s:%d", \
            self.user, self.passwd, self.host, self.port)
        self.ctxStack = ContextStack(self.log)
        
    def __del__(self):
        self.log.debug("destroyed %s:%s@%s:%d", \
            self.user, self.passwd, self.host, self.port)
        pass
        
    def __str__(self):
        return "CLIConnection [%08X]: %s:%s@%s:%d" % \
            (self.id, self.user, self.passwd, self.host, self.port) 
    
#    def recvResponse(self):
#        arr = array.array('c')
#        buf = ""
#        eor = False
#        while not eor:
#            try:
#                buf = self.sock.recv(self.BUFF_SZ)
#                eor = len(buf) < self.BUFF_SZ
##                eor = len(buf) == 0
#            except socket.timeout:
#                eor = True
#            arr.extend(buf)
#        s = arr.tostring()
#        self.log.log(LEVEL_PROTO, "received(%u): {\n%s\n}", len(s), s)
#        return s

    def recvResponse(self):
        nb = self.sock.recv_into(self.buf, self.BUFF_SZ)
        if nb == 0:
            self.log.warning("connection closed by server")
            return None
        if sys.version_info[0] < 3:
          self.buf[nb] = '\0'
        else:
          self.buf[nb] = u'\0'
        s = self.buf.tostring()[0:nb]
        self.log.log(LEVEL_PROTO, "received(%u): {\n%s\n}", nb, s)
        if sys.version_info[0] < 3:
          return s
        else:
          return s.decode('utf-8')
  
    def sendRequest(self, req):
        nb = len(req) + 2
        reqCRLF=req + "\r\n"
        if sys.version_info[0] >= 3:
          reqCRLF=str.encode(reqCRLF)
        nbw = self.sock.send(reqCRLF)
        if nb != nbw:
            self.log.error("send error (%d/%d)", nbw, nb)
            raise exceptions.IOError()
        self.log.log(LEVEL_PROTO, "sent(%u): {\n%s\n}", nb, req)
    
    def login(self):
        ''' connect '''
        if self.stat != self.STAT_NEW:
            raise exceptions.RuntimeError()
        self.sock.connect((self.host, self.port))
        self.log.info("connected to %s:%d", \
            self.host, self.port)
        ''' check initial message with login prompt '''
        res = self.recvResponse()
        mo = PattLoginResponse.match(res)
        if not mo or mo.group('ctx') != 'login':
            raise clierror.InvalidResponseError()
        ''' send user name '''
        self.sendRequest("user " + self.user)
        ''' check password prompt '''
        res = self.recvResponse()
        mo = PattPasswordResponse.match(res)
        if not mo or mo.group('ctx') != 'password':
            raise clierror.InvalidResponseError()
        ''' send user password '''
        self.sendRequest(self.passwd)
        ''' check root prompt '''
        res = self.recvResponse()
        mo = PattRootResponse.match(res)
        if not mo or mo.group('ctx') != '#':
            raise clierror.InvalidResponseError()
        self.ctxStack.push('#')
        self.stat = self.STAT_LOGGEDIN
        self.log.info("logged-in as '%s'", self.user)

    def logout(self):
        if self.stat != self.STAT_LOGGEDIN: 
            raise exceptions.RuntimeError()
        self.sendRequest("quit")
        self.sock.close()
        self.stat = self.STAT_LOGGEDOUT
        self.log.info("logged-out as '%s'", self.user)
        
    def navigateToServer(self):
        pass
    
    def navigateToDomain(self, domName):
        pass
    
    def navigateToAccount(self, acctName, domName=None):
        pass

