#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 Axigen Messaging SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" Definitions of classes related to CLI Command Handlers
    CommandHandler
      MacroHandler      
      ListHandler
        NameListHandler
      ShowHandler
        NameValueMapHandler
      ShowRegistryHandler
        RegistryNameValueMapHandler
"""

__copyright__ = "Copyright (C) 2020, Axigen Messaging SRL"
__license__ = "Apache License Version 2.0"
__version__ = "1.1.0"
__author__ = 'eugen.belea@axigen.com (Eugen Belea)'
__maintainer__ = 'eugen.belea@axigen.com (Eugen Belea)'

try:
    import exceptions
except ImportError:
    import builtins as exceptions
import re

PattNameList = re.compile('^(?P<name>\w+[\w\-\.]*)(\s+(?P<val>.*))?')
#PattNameValuePair1 = re.compile('^(?P<name>\w+[\w\-]*)\s+\=\s+(?P<val>[\w\-\ \"\*\.]+)')
PattNameValuePair = re.compile(r"""
^(?P<name>[a-zA-Z](\-?\w+)*)
\ +=\ +
(?P<val>([\w\-]+)|(\"([ \!\#-\~]|(\\\"))*\"?))
""", re.VERBOSE)
PattRegNameValuePair = re.compile(r"""
^(?P<name>[a-zA-Z]+(\ *\w+)*)
\ +=\ +
(?P<val>[a-zA-Z0-9]+[\w\ \.\,\:\;\+\-\=\(\)]*)
""", re.VERBOSE)
PattPrefixValue = re.compile(r"""
^(?P<val>[^\ ]+).*
""", re.VERBOSE)

class CommandHandler():
    '''
    CLI Command Handler (base class of all command handlers)
    Handles the response to a command
    '''
    def __init__(self):
        self.log = None

    def setLog(self, _log=None):
        self.log = _log
        
    def handleError(self, errMsg):
        raise exceptions.NotImplementedError()
    
    def handleOk(self, okMsg):
        raise exceptions.NotImplementedError()
    
    def handleLine(self, line, lineNo):        
        raise exceptions.NotImplementedError()

class MacroHandler(CommandHandler):
    def __init__(self):
        self.log = None

    def setLog(self, _log=None):
        self.log = _log
        
    def handleCommandBegin(self, cmdNo, cmdName):
        raise exceptions.NotImplementedError()
    
    def handleCommandEnd(self, cmdNo, cmdName):
        raise exceptions.NotImplementedError()
  
class ListHandler(CommandHandler):
    def __init__(self):
        CommandHandler.__init__(self)
    
    def handleError(self, errMsg):
        self.log.debug("ListHandler: ERROR")
    
    def handleOk(self, okMsg):
        self.log.debug("ListHandler: OK")
    
    def handleLine(self, line, lineNo):
        ''' Skip 5 heading lines '''        
        if lineNo <= 5:
            return
        mo = PattNameList.match(line)
        if mo:
            self.log.debug("ListHandler: name '%s'", mo.group('name'))
            self.handleElement(mo.group('name'), mo.group('val'))
        else:
#            self.log.warning("ListHandler: line not matching name")
            pass
    
    def handleElement(self, name, value):
        raise exceptions.NotImplementedError()

class NameListHandler(ListHandler):
    def __init__(self):
        ListHandler.__init__(self)
        self.list = []
    
    def handleElement(self, name, value):
        if name == '':
            self.log.warning("NameList: null name received")
            return
        if name in self.list:
            self.log.warning("NameList: name '%s' already existing", name)
            return
        self.list.append(name)

class ShowHandler(CommandHandler):
    def __init__(self, _wsAttrName=False, _inheritedFlag=False):
        CommandHandler.__init__(self)
        self.wsAttrName = _wsAttrName
        self.inheritedFlag = _inheritedFlag
    
    def handleError(self, errMsg):
        self.log.debug("ShowHandler: ERROR")
    
    def handleOk(self, okMsg):
        self.log.debug("ShowHandler: OK")
    
    def handleLine(self, line, lineNo):
        mo = PattNameValuePair.match(line)
        if mo:
#            self.log.debug("ShowHandler: attribute: name '%s' value '%s'", \
#                mo.group('name'), mo.group('val'))
            v = mo.group('val')
            if v and v[0] == '"':
                v = v[1:-1]
            self.handleNameValuePair(mo.group('name'), v, False)
        else:
#            self.log.warning("ShowHandler: line not matching name-value pair")
            pass
    
    def handleNameValuePair(self, name, value, isInherited=False):
        raise exceptions.NotImplementedError()

class ShowRegistryHandler(CommandHandler):
    def __init__(self):
        CommandHandler.__init__(self)
    
    def handleError(self, errMsg):
        self.log.debug("ShowRegistryHandler: ERROR")
    
    def handleOk(self, okMsg):
        self.log.debug("ShowRegistryHandler: OK")
    
    def handleLine(self, line, lineNo):
        mo = PattRegNameValuePair.match(line)
        if mo:
            self.log.debug("ShowHandler: attribute: name '%s' value '%s'", \
                mo.group('name'), mo.group('val'))
            self.handleNameValuePair(mo.group('name'), mo.group('val'), False)
        else:
            self.log.warning("ShowHandler: '%s'", line)            
            self.log.warning("ShowHandler: line not matching name-value pair")
            pass
    
    def handleNameValuePair(self, name, value, isInherited=False):
        raise exceptions.NotImplementedError()


class NameValueMapHandler(ShowHandler):
    def __init__(self, _selectedNames=None, _wsAttrName=False, _inheritedFlag=False):
        ShowHandler.__init__(self)
        self.selectedNames = _selectedNames
        self.map = {}

    def __del__(self):
        del self.map

    def handleNameValuePair(self, name, value, isInherited):
        if not self.selectedNames or (name in self.selectedNames):
            self.log.debug("NameValueMap: attribute: name '%s' value '%s'", \
                name, value)
            self.map[name] = value

class RegistryNameValueMapHandler(ShowRegistryHandler):
    def __init__(self, _selectedNames=None):
        ShowRegistryHandler.__init__(self)
        self.selectedNames = _selectedNames
        self.map = {}

    def __del__(self):
        del self.map

    def handleNameValuePair(self, name, value, isInherited):
        if not self.selectedNames:
            self.log.debug("NameValueMap: attribute: name '%s' value '%s'", \
                name, value)
            self.map[name] = value
        elif self.selectedNames.has_key(name):
            self.log.debug("NameValueMap: attribute: name '%s' value '%s'", \
                name, value)
            if self.selectedNames[name]: # Full value
                self.map[name] = value
            else: # Only prefix as value
                mv = PattPrefixValue.match(value)
                self.map[name] = mv.group('val')
