#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 Axigen Messaging SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" Definitions of classes related to CLI Context Navigation
    NavCommand
      ServerNavCommand
      DomainNavCommand
      AccountNavCommand
"""

__copyright__ = "Copyright (C) 2020, Axigen Messaging SRL"
__license__ = "Apache License Version 2.0"
__version__ = "1.1.0"
__author__ = 'eugen.belea@axigen.com (Eugen Belea)'
__maintainer__ = 'eugen.belea@axigen.com (Eugen Belea)'


from axitools.context import *

class NavCommand():
    '''
    CLI Navigation Command (base class for all navigation commands)
    '''
    def __init__(self):
        pass
    
    def navigate(self, conn):
        '''
        This method should be implemented by direct subclasses
        It should perform communication with server for the navigation command
        '''
        raise exception.NotImplementedError

    def __doCommunicate(self, conn):
        '''
        This method should be implemented by subclasses
        It should perform communication with server for the navigation command
        '''
        raise exception.NotImplementedError





class ServerNavCommand(NavCommand):
    '''
    CLI Command for Navigation to the Server context 
    '''
    def __init__(self):
        super().__init__(self)
    
    def navigate(self, conn):
        topCtx = conn.ctxStack.front()
        if not isinstance(topCtx, RootContext):
            raise InvalidContextError
        # execute navigation: config server
        conn.ctxStack.push(ServerContext())


class DomainNavCommand(NavCommand):
    '''
    CLI Command for Navigation to a Domain context 
    '''
    def __init__(self, _domName):
        super().__init__(self)
        self.domName = _domName

    def navigate(self, conn):
        topCtx = conn.ctxStack.front()
        if not isinstance(topCtx, RootContext):
            raise InvalidContextError
        # execute navigation: update domain name <domName>
        conn.ctxStack.push(DomainContext(self.domName))

class AccountNavCommand(NavCommand):
    '''
    CLI Command for Navigation to an Account context 
    '''
    def __init__(self, _acctName):
        super().__init__(self)
        self.acctName = _acctName

    def navigate(self, conn):
        topCtx = conn.ctxStack.front()
        if not isinstance(topCtx, DomainContext):
            raise InvalidContextError
        # execute navigation: update account name <acctName>
        conn.ctxStack.push(AccountContext(self.acctName))
