
from __future__ import print_function

from . import bkengine
import sys
import time

hasTTY = False
hasCurses = False
try:
    if sys.stdout.isatty():
        hasTTY = True
        import curses
        hasCurses = True
except:
    pass

def _sizestr(fileSize):
    kb = 1024
    mb = kb * 1024
    gb = mb * 1024
    if fileSize >= gb:
        sz = "{0}.{1:02} GB".format(fileSize // gb, ((fileSize * 100) // gb) % 100)
    elif fileSize >= mb:
        sz = "{0}.{1:02} MB".format(fileSize // mb, ((fileSize * 100) // mb) % 100)
    elif fileSize >= kb:
        sz = "{0}.{1:02} KB".format(fileSize // kb, ((fileSize * 100) // kb) % 100)
    else:
        sz = "{0} B".format(fileSize)
    return sz

class CursesProgressIndicator(bkengine.ProgressIndicator):
    def __init__(self, cursesWindow = None):
        bkengine.ProgressIndicator.__init__(self)
        self._wobj = cursesWindow
        self._globalDisabled = False
        self._errors = []
        self._restoreMode = False

    def beginBackup(self, srcRoot, destRoot, restoreMode):
        self._restoreMode = restoreMode
        if not self._wobj:
            self._wobj = curses.initscr()
            self._endcurses = True
        else:
            self._endcurses = False
        self._wobj.clear()
        if self._restoreMode:
            self._wobj.addstr(1, 0, "Restoring '{0}' from '{1}'".format(destRoot.getPath()[-30:], srcRoot.getPath()[-30:]))
        else:
            self._wobj.addstr(1, 0, "Backup '{0}' into '{1}'".format(srcRoot.getPath()[-30:], destRoot.getPath()[-30:]))
        self._wobj.move(10, 0)
        self._wobj.refresh()
        self._starttime = time.time()

    def endBackup(self, successfull, copyFileCount, copyFileSize):
        if self._endcurses:
            curses.endwin()
        else:
            self._wobj.clear()
        sz = _sizestr(copyFileSize)
        elapsed = time.time() - self._starttime
        if elapsed < 0:
            elapsed = 0
        for err in self._errors:
            print(err)
        if successfull:
            if self._restoreMode:
                print("Restore completed ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))
            else:
                print("Backup completed ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))
        else:
            if self._restoreMode:
                print("Resore failed ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))
            else:
                print("Backup failed ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))

    def abortBackup(self, copyFileCount, copyFileSize):
        if self._endcurses:
            curses.endwin()
        else:
            self._wobj.clear()
        sz = _sizestr(copyFileSize)
        elapsed = time.time() - self._starttime
        if elapsed < 0:
            elapsed = 0
        if self._restoreMode:
            print("Restore aborted by user ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))
        else:
            print("Backup aborted by user ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))

    def reportGlobalProgress(self, backupProgress, backupCount, copyFileCount, copyFileSize):
        if self._globalDisabled:
            return
        sz = _sizestr(copyFileSize)
        if backupCount > 0:
            percent = (backupProgress * 100) / backupCount
        else:
            percent = 0

        self._wobj.hline(3, 0, ' ', self._wobj.getmaxyx()[1])
        self._wobj.addstr(3, 1, "{0} of {1} ({2}%)".format(backupProgress, backupCount, percent))
        self._wobj.addstr(3, 23, "{0} in {1} files".format(sz, copyFileCount))
        self._wobj.addstr(3, 50, '#' * (percent / 4))
        self._wobj.move(10, 0)
        self._wobj.refresh()

    def beginDomainBackup(self, backupDomain, domainIdx, domainCount):
        self._wobj.addstr(5, 1, backupDomain.getName()[:20])
        self._wobj.addstr(5, 23, "scanning")
        self._wobj.move(10, 0)
        self._wobj.refresh()
        #percent = (domainIdx * 100) / domainCount;
        #progress="{0} of {1} ({2}%)".format(domainIdx, domainCount, percent)
        #for line in range(2, 6):
        #    self._wobj.hline(line, 1, ' ', 80)
        #self._wobj.addstr(2, 1, "{0:29}: {1}".format(progress, backupDomain.getName()))
        #self._wobj.move(6, 1)
        #self._wobj.refresh()
        
    def endDomainBackup(self, backupDomain, successfull, domainIdx, domainCount):
        self._wobj.hline(5, 0, ' ', self._wobj.getmaxyx()[1])
        self._wobj.move(10, 0)
        self._wobj.refresh()
        
    def beginDomainObjectBackup(self, backupDomainObject, domainObjectIdx, domainObjectCount):
        percent = (domainObjectIdx * 100) / domainObjectCount;
        progress="{0} of {1} ({2}%)".format(domainObjectIdx, domainObjectCount, percent)
        self._wobj.addstr(5, 23, progress)
        self._wobj.addstr(5, 50, '#' * (percent / 4))
        self._wobj.addstr(6, 1, backupDomainObject.getName()[:20])
        self._wobj.addstr(6, 23, "scanning")
        self._wobj.move(10, 0)
        self._wobj.refresh()
        
    def endDomainObjectBackup(self, backupDomainObject, successfull, domainObjectIdx, domainObjectCount):
        self._wobj.hline(6, 0, ' ', self._wobj.getmaxyx()[1])
        self._wobj.move(10, 0)
        self._wobj.refresh()

    def beginFolderBackup(self, backupFolder, folderIdx, folderCount):
        percent = (folderIdx * 100) / folderCount;
        progress="{0} of {1} ({2}%)".format(folderIdx, folderCount, percent)
        self._wobj.addstr(6, 23, progress)
        self._wobj.addstr(6, 50, '#' * (percent / 4))
        self._wobj.addstr(7, 1, backupFolder.getName()[:20])
        self._wobj.addstr(7, 23, "scanning")
        self._wobj.move(10, 0)
        self._wobj.refresh()
        self._mailProgress = -1
        
    def endFolderBackup(self, backupFolder, successfull, folderIdx, folderCount):
        self._globalDisabled = False
        self._wobj.hline(7, 0, ' ', self._wobj.getmaxyx()[1])
        self._wobj.move(10, 0)
        self._wobj.refresh()
        
    def beginMailBackup(self, backupFolder, mailIdx, mailCount):
        percent = (mailIdx * 100) / mailCount;
        if (percent == self._mailProgress) and ((mailIdx % 50) != 0):
            self._globalDisabled = True
            return
        self._globalDisabled = False
        self._mailProgress = percent
        progress="{0} of {1} ({2}%)".format(mailIdx, mailCount, percent)
        self._wobj.addstr(7, 23, progress)
        self._wobj.addstr(7, 50, '#'*(percent/4))
        self._wobj.move(10, 0)
        self._wobj.refresh()

    def endMailBackup(self, backupFolder, successfull, mailIdx, mailCount):
        """
        Called when backup operation is finalized for a mail.
        """
        pass

    def reportError(self, errorDetails):
        self._errors.append(errorDetails)
        
class TextProgressIndicator(bkengine.ProgressIndicator):
    def __init__(self):
        bkengine.ProgressIndicator.__init__(self)
        self._restoreMode = False

    def beginBackup(self, srcRoot, destRoot, restoreMode):
        self._restoreMode = restoreMode
        if self._restoreMode:
            print("Begin restoring '{0}' from '{1}'".format(destRoot.getPath(), srcRoot.getPath()))
        else:
            print("Begin backup '{0}' into '{1}'".format(srcRoot.getPath(), destRoot.getPath()))
        self._starttime = time.time()

    def endBackup(self, successfull, copyFileCount, copyFileSize):
        sz = _sizestr(copyFileSize)
        elapsed = time.time() - self._starttime
        if elapsed < 0:
            elapsed = 0
        if successfull:
            if self._restoreMode:
                print("Restore completed ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))
            else:
                print("Backup completed ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))
        else:
            if self._restoreMode:
                print("Restore failed ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))
            else:
                print("Backup failed ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))

    def abortBackup(self, copyFileCount, copyFileSize):
        sz = _sizestr(copyFileSize)
        elapsed = time.time() - self._starttime
        if elapsed < 0:
            elapsed = 0
        if self._restoreMode:
            print("Restore aborted by user ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))
        else:
            print("Backup aborted by user ({0} in {1} files copied in {2:.2f} seconds)".format(sz, copyFileCount, elapsed))

    def reportGlobalProgress(self, backupProgress, backupCount, copyFileCount, copyFileSize):
        pass
        #print("Global progress: {0} of {1} ({2} bytes in {3} files)".format(backupProgress, backupCount, copyFileSize, copyFileCount))
        
    def beginDomainBackup(self, backupDomain, domainIdx, domainCount):
        print("Begin domain:", backupDomain.getName(), "(", domainIdx, "of", domainCount, ")")
        
    def endDomainBackup(self, backupDomain, successfull, domainIdx, domainCount):
        print("End domain:", backupDomain.getName())
        
    def beginDomainObjectBackup(self, backupDomainObject, domainObjectIdx, domainObjectCount):
        print("  Begin domain object:", backupDomainObject.getName(), "(", domainObjectIdx, "of", domainObjectCount, ")")
        
    def endDomainObjectBackup(self, backupDomainObject, successfull, domainObjectIdx, domainObjectCount):
        print("  End domain object:", backupDomainObject.getName())

    def beginFolderBackup(self, backupFolder, folderIdx, folderCount):
        print("    Begin folder:", backupFolder.getName(), "(", folderIdx, "of", folderCount, ")")
        self._mailProgress = -1
        
    def endFolderBackup(self, backupFolder, successfull, folderIdx, folderCount):
        if self._mailProgress != -1:
            print("{0}\r".format(' '*70),)
        print("    End folder:", backupFolder.getName())
        
    def beginMailBackup(self, backupFolder, mailIdx, mailCount):
        if not hasTTY:
            return
        percent = (mailIdx * 100) / mailCount;
        if percent == self._mailProgress:
            return
        self._mailProgress = percent        
        hashProgress='#'*(percent/3)
        progress="{0} of {1} ({2}%)".format(mailIdx, mailCount, percent)
        print("    {0:23}: {1}\r".format(progress, hashProgress),)

    def endMailBackup(self, backupFolder, successfull, mailIdx, mailCount):
        """
        Called when backup operation is finalized for a mail.
        """
        pass
    
    def reportError(self, errorDetails):
        print(errorDetails)

def getProgressIndicator(allowCurses = True):
    if allowCurses and hasCurses:
        return CursesProgressIndicator()
    return TextProgressIndicator()

def getCursesProgressIndicator(cursesWindow):
    return CursesProgressIndicator(cursesWindow)
